/******************************************************************************
 * jWebUnit project (http://jwebunit.sourceforge.net)                         *
 * Distributed open-source, see full license under LICENCE.txt                *
 ******************************************************************************/
package net.sourceforge.jwebunit.junit;

import java.io.File;
import java.io.PrintStream;
import java.net.URL;

import net.sourceforge.jwebunit.api.IJWebUnitDialog;
import net.sourceforge.jwebunit.html.Table;
import net.sourceforge.jwebunit.util.TestContext;

import junit.framework.TestCase;

/**
 * Superclass for Junit TestCases which provides web application navigation and 
 * JUnit assertions. This class uses and is generated from {@link net.sourceforge.jwebunit.junit.WebTester}.
 * 
 * @author JavaCC
 */
public class WebTestCase extends TestCase {
    private WebTester tester = null;

    public WebTestCase(String name) {
        super(name);
    }

    public WebTestCase() {
        super();
    }

    public void tearDown() throws Exception {
        closeBrowser();
        super.tearDown();
    }

    /**
     * Clean up unused memory. Using <tt>setUp</tt> and <tt>tearDown</tt> is
     * not an option for this requires the subclasses of this class to call the
     * respective <tt>super</tt> methods.
     *
     * Original patch contributed by Budi Boentaran.
     */
    public void runBare() throws Throwable {
        try {
            tester=new WebTester();
            super.runBare();
        } finally {
            tester=null;
        }
    }

    /**
     * Provides access to the testing engine for subclasses - in case functionality not yet wrappered required
     * by test.
     * 
     * If the dialog is not explicitly set the jWebUnit framework will default to using the orignal testing engine,
     * which is, htmlunit.
     * 
     * @return IJWebUnitDialog instance used to wrapper htmlunit conversation.
     * @deprecated You should not use plugin specific fonctionality. Please ask for a new core feature instead.
     */
    public IJWebUnitDialog getDialog() {
        return tester.getDialog();
    }

    /**
     * Close the current conversation.
     */
    public void closeBrowser() {
        tester.closeBrowser();
    }

    /**
     * Close the current window.
     */
    public void closeWindow() {
        tester.closeWindow();
    }

    /**
     * Set the testing engine.
     * @param aIJWebUnitDialog Testing engine.
     */
    public void setDialog(IJWebUnitDialog aIJWebUnitDialog) {
        tester.setDialog(aIJWebUnitDialog);
    }

    /**
     * Provide access to test testContext.
     * 
     * @return TestContext
     */
    public TestContext getTestContext() {
        return tester.getTestContext();
    }

    /**
     * Allows setting an external test testContext class that might be extended from TestContext. Example:
     * setTestContext(new CompanyATestContext());
     * 
     * CompanyATestContext extends TestContext.
     * 
     * @param aTestContext
     */
    public void setTestContext(TestContext aTestContext) {
        tester.setTestContext(aTestContext);
    }

    /**
     * Begin conversation at a URL absolute or relative to base URL. Use
     * {@link TestContext#setBaseUrl(String) getTestContext().setBaseUrl(String)} to define base URL. Absolute URL
     * should start with "http://", "https://" or "www.".
     * 
     * @param url absolute or relative URL (relative to base URL).
     */
    public void beginAt(String aRelativeURL) {
        tester.beginAt(aRelativeURL);
    }

    /**
     * Return the value of a web resource based on its key. This translates to a property file lookup with the locale
     * based on the current TestContext.
     * 
     * @param key name of the web resource.
     * @return value of the web resource, encoded according to TestContext.
     */
    public String getMessage(String key) {
        return tester.getMessage(key);
    }

    /**
     * Assert title of current html page in conversation matches an expected value.
     * 
     * @param title expected title value
     */
    public void assertTitleEquals(String title) {
        tester.assertTitleEquals(title);
    }

    /**
     * Assert title of current html page in conversation matches an expected regexp.
     * 
     * @param regexp expected title regexp
     */
    public void assertTitleMatch(String regexp) {
        tester.assertTitleMatch(regexp);
    }

    /**
     * Assert title of current html page matches the value of a specified web resource.
     * 
     * @param titleKey web resource key for title
     */
    public void assertTitleEqualsKey(String titleKey) {
        tester.assertTitleEqualsKey(titleKey);
    }

    /**
     * Assert that a web resource's value is present.
     * 
     * @param key web resource name
     */
    public void assertKeyPresent(String key) {
        tester.assertKeyPresent(key);
    }

    /**
     * Assert that supplied text is present.
     * 
     * @param text
     */
    public void assertTextPresent(String text) {
        tester.assertTextPresent(text);
    }

    /**
     * Assert that supplied regexp is matched.
     * 
     * @param regexp
     */
    public void assertMatch(String regexp) {
        tester.assertMatch(regexp);
    }

    /**
     * Assert that a web resource's value is not present.
     * 
     * @param key web resource name
     */
    public void assertKeyNotPresent(String key) {
        tester.assertKeyNotPresent(key);
    }

    /**
     * Assert that supplied text is not present.
     * 
     * @param text
     */
    public void assertTextNotPresent(String text) {
        tester.assertTextNotPresent(text);
    }

    /**
     * Assert that supplied regexp is not present.
     * 
     * @param regexp
     */
    public void assertNoMatch(String regexp) {
        tester.assertNoMatch(regexp);
    }

    /**
     * 
     * @param tableSummaryNameOrId
     * @return Object that represent a html table in a way independent from plugin.
     */
    public Table getTable(String tableSummaryNameOrId) {
        return tester.getTable(tableSummaryNameOrId);
    }

    /**
     * Assert that a table with a given summary or id value is present.
     * 
     * @param tableSummaryNameOrId summary, name or id attribute value of table
     */
    public void assertTablePresent(String tableSummaryNameOrId) {
        tester.assertTablePresent(tableSummaryNameOrId);
    }

    /**
     * Assert that a table with a given summary or id value is not present.
     * 
     * @param tableSummaryNameOrId summary, name or id attribute value of table
     */
    public void assertTableNotPresent(String tableSummaryNameOrId) {
        tester.assertTableNotPresent(tableSummaryNameOrId);
    }

    /**
     * Assert that the value of a given web resource is present in a specific table.
     * 
     * @param tableSummaryOrId summary or id attribute value of table
     * @param key web resource name
     */
    public void assertKeyInTable(String tableSummaryOrId, String key) {
        tester.assertKeyInTable(tableSummaryOrId, key);
    }

    /**
     * Assert that supplied text is present in a specific table.
     * 
     * @param tableSummaryNameOrId summary, name or id attribute value of table
     * @param text
     */
    public void assertTextInTable(String tableSummaryNameOrId, String text) {
        tester.assertTextInTable(tableSummaryNameOrId, text);
    }

    /**
     * Assert that supplied regexp is matched in a specific table.
     * 
     * @param tableSummaryNameOrId summary, name or id attribute value of table
     * @param regexp
     */
    public void assertMatchInTable(String tableSummaryNameOrId, String regexp) {
        tester.assertMatchInTable(tableSummaryNameOrId, regexp);
    }

    /**
     * Assert that the values of a set of web resources are all present in a specific table.
     * 
     * @param tableSummaryOrId summary, name or id attribute value of table
     * @param keys Array of web resource names.
     */
    public void assertKeysInTable(String tableSummaryOrId, String[] keys) {
        tester.assertKeysInTable(tableSummaryOrId, keys);
    }

    /**
     * Assert that a set of text values are all present in a specific table.
     * 
     * @param tableSummaryOrId summary, name or id attribute value of table
     * @param text Array of expected text values.
     */
    public void assertTextInTable(String tableSummaryOrId, String[] text) {
        tester.assertTextInTable(tableSummaryOrId, text);
    }

    /**
     * Assert that a set of regexp values are all matched in a specific table.
     * 
     * @param tableSummaryOrId summary, name or id attribute value of table
     * @param text Array of expected regexps to match.
     */
    public void assertMatchInTable(String tableSummaryOrId, String[] regexp) {
        tester.assertMatchInTable(tableSummaryOrId, regexp);
    }

    /**
     * Assert that the value of a given web resource is not present in a specific table.
     * 
     * @param tableSummaryOrId summary, name or id attribute value of table
     * @param key web resource name
     */
    public void assertKeyNotInTable(String tableSummaryOrId, String key) {
        tester.assertKeyNotInTable(tableSummaryOrId, key);
    }

    /**
     * Assert that supplied text is not present in a specific table.
     * 
     * @param tableSummaryNameOrId summary, name or id attribute value of table
     * @param text
     */
    public void assertTextNotInTable(String tableSummaryNameOrId, String text) {
        tester.assertTextNotInTable(tableSummaryNameOrId, text);
    }

    /**
     * Assert that none of a set of text values are present in a specific table.
     * 
     * @param tableSummaryNameOrId summary, name or id attribute value of table
     * @param text Array of text values
     */
    public void assertTextNotInTable(String tableSummaryNameOrId, String[] text) {
        tester.assertTextNotInTable(tableSummaryNameOrId, text);
    }

    /**
     * Assert that supplied regexp is not present in a specific table.
     * 
     * @param tableSummaryNameOrId summary, name or id attribute value of table
     * @param text
     */
    public void assertNoMatchInTable(String tableSummaryNameOrId, String regexp) {
        tester.assertNoMatchInTable(tableSummaryNameOrId, regexp);
    }

    /**
     * Assert that none of a set of regexp values are present in a specific table.
     * 
     * @param tableSummaryNameOrId summary, name or id attribute value of table
     * @param text Array of text values
     */
    public void assertNoMatchInTable(String tableSummaryNameOrId, String[] regexp) {
        tester.assertNoMatchInTable(tableSummaryNameOrId, regexp);
    }

    /**
     * Assert that a specific table matches an ExpectedTable.
     * 
     * @param tableSummaryNameOrId summary, name or id attribute value of table
     * @param expectedTable represents expected values (colspan supported).
     */
    public void assertTableEquals(String tableSummaryNameOrId, Table expectedTable) {
        tester.assertTableEquals(tableSummaryNameOrId, expectedTable);
    }

    /**
     * Assert that a specific table matches a matrix of supplied text values.
     * 
     * @param tableSummaryNameOrId summary, name or id attribute value of table
     * @param expectedCellValues double dimensional array of expected values
     */
    public void assertTableEquals(String tableSummaryNameOrId, String[][] expectedCellValues) {
        tester.assertTableEquals(tableSummaryNameOrId, expectedCellValues);
    }

    /**
     * Assert that a range of rows for a specific table matches a matrix of supplied text values.
     * 
     * @param tableSummaryNameOrId summary, name or id attribute value of table
     * @param startRow index of start row for comparison
     * @param expectedTable represents expected values (colspan and rowspan supported).
     */
    public void assertTableRowsEqual(String tableSummaryNameOrId, int startRow, Table expectedTable) {
        tester.assertTableRowsEqual(tableSummaryNameOrId, startRow, expectedTable);
    }

    /**
     * Assert that a range of rows for a specific table matches a matrix of supplied text values.
     * 
     * @param tableSummaryNameOrId summary, name or id attribute value of table
     * @param startRow index of start row for comparison
     * @param expectedTable represents expected values (colspan and rowspan supported).
     */
    public void assertTableRowsEqual(String tableSummaryNameOrId, int startRow, String[][] expectedTable) {
        tester.assertTableRowsEqual(tableSummaryNameOrId, startRow, expectedTable);
    }

    /**
     * Assert that the number of rows for a specific table equals expected value.
     * 
     * @param tableSummaryNameOrId summary, name or id attribute value of table
     * @param expectedRowCount expected row count.
     */
    public void assertTableRowCountEquals(String tableSummaryNameOrId, int expectedRowCount) {
        tester.assertTableRowCountEquals(tableSummaryNameOrId, expectedRowCount);
    }

    /**
     * Assert that a specific table matches an ExpectedTable.
     * 
     * @param tableSummaryOrId summary or id attribute value of table
     * @param expectedTable represents expected regexps (colspan supported).
     */
    public void assertTableMatch(String tableSummaryOrId, Table expectedTable) {
        tester.assertTableMatch(tableSummaryOrId, expectedTable);
    }

    /**
     * Assert that a specific table matches a matrix of supplied regexps.
     * 
     * @param tableSummaryOrId summary or id attribute value of table
     * @param expectedCellValues double dimensional array of expected regexps
     */
    public void assertTableMatch(String tableSummaryOrId, String[][] expectedCellValues) {
        tester.assertTableMatch(tableSummaryOrId, expectedCellValues);
    }

    /**
     * Assert that a range of rows for a specific table matches a matrix of supplied regexps.
     * 
     * @param tableSummaryOrId summary or id attribute value of table
     * @param startRow index of start row for comparison
     * @param expectedTable represents expected regexps (colspan and rowspan supported).
     */
    public void assertTableRowsMatch(String tableSummaryOrId, int startRow, Table expectedTable) {
        tester.assertTableRowsMatch(tableSummaryOrId, startRow, expectedTable);
    }

    /**
     * Assert that a range of rows for a specific table matches a matrix of supplied regexps.
     * 
     * @param tableSummaryOrId summary or id attribute value of table
     * @param startRow index of start row for comparison
     * @param expectedTable represents expected regexps (colspan and rowspan not supported).
     */
    public void assertTableRowsMatch(String tableSummaryOrId, int startRow, String[][] expectedTable) {
        tester.assertTableRowsMatch(tableSummaryOrId, startRow, expectedTable);
    }

    /**
     * Assert that a form input element with a given name is present.
     * 
     * @param formElementName
     */
    public void assertFormElementPresent(String formElementName) {
        tester.assertFormElementPresent(formElementName);
    }

    /**
     * Assert that a form input element with a given name is not present.
     * 
     * @param formElementName
     */
    public void assertFormElementNotPresent(String formElementName) {
        tester.assertFormElementNotPresent(formElementName);
    }

    /**
     * Assert that a form checkbox with a given name is present.
     * 
     * @param checkboxName checkbox name.
     */
    public void assertCheckboxPresent(String checkboxName) {
        tester.assertCheckboxPresent(checkboxName);
    }

    /**
     * Assert that a given checkbox is present.
     * 
     * @param checkboxName checkbox name attribut.
     * @param checkboxValue checkbox value attribut.
     */
    public void assertCheckboxPresent(String checkboxName, String checkboxValue) {
        tester.assertCheckboxPresent(checkboxName, checkboxValue);
    }

    /**
     * Assert that a form checkbox with a given name is not present.
     * 
     * @param checkboxName checkbox name.
     */
    public void assertCheckboxNotPresent(String checkboxName) {
        tester.assertCheckboxNotPresent(checkboxName);
    }

    /**
     * Assert that a given checkbox is not present.
     * 
     * @param checkboxName checkbox name.
     * @param checkboxValue checkbox value attribut.
     */
    public void assertCheckboxNotPresent(String checkboxName, String checkboxValue) {
        tester.assertCheckboxNotPresent(checkboxName, checkboxValue);
    }

    /**
     * Assert that there is a form present.
     * 
     */
    public void assertFormPresent() {
        tester.assertFormPresent();
    }

    /**
     * Assert that there is a form with the specified name or id present.
     * 
     * @param nameOrID
     */
    public void assertFormPresent(String nameOrID) {
        tester.assertFormPresent(nameOrID);
    }

    /**
     * Assert that there is not a form present.
     * 
     */
    public void assertFormNotPresent() {
        tester.assertFormNotPresent();
    }

    /**
     * Assert that there is not a form with the specified name or id present.
     * 
     * @param nameOrID
     */
    public void assertFormNotPresent(String nameOrID) {
        tester.assertFormNotPresent(nameOrID);
    }

    /**
     * Assert that a specific form element has an expected value. Can be used to check hidden input.
     * 
     * @param formElementName
     * @param expectedValue
     * @deprecated
     */
    public void assertFormElementEquals(String formElementName, String expectedValue) {
        tester.assertFormElementEquals(formElementName, expectedValue);
    }

    /**
     * Assert that a specific form element matches an expected regexp.
     * 
     * @param formElementName
     * @param regexp
     * @deprecated
     */
    public void assertFormElementMatch(String formElementName, String regexp) {
        tester.assertFormElementMatch(formElementName, regexp);
    }

    /**
     * Assert that a form element had no value / is empty.
     * 
     * @param formElementName
     * @deprecated
     */
    public void assertFormElementEmpty(String formElementName) {
        tester.assertFormElementEmpty(formElementName);
    }

    /**
     * Assert that an input text element with name <code>formElementName</code> has the <code>expectedValue</code>
     * value.
     * 
     * @param formElementName the value of the name attribute of the element
     * @param expectedValue the expected value of the given input element
     */
    public void assertTextFieldEquals(String formElementName, String expectedValue) {
        tester.assertTextFieldEquals(formElementName, expectedValue);
    }

    /**
     * Assert that an input hidden element with name <code>formElementName</code> has the <code>expectedValue</code>
     * value.
     * 
     * @param formElementName the value of the name attribute of the element
     * @param expectedValue the expected value of the given input element
     */
    public void assertHiddenFieldPresent(String formElementName, String expectedValue) {
        tester.assertHiddenFieldPresent(formElementName, expectedValue);
    }

    /**
     * Assert that a specific checkbox is selected.
     * 
     * @param checkBoxName
     */
    public void assertCheckboxSelected(String checkBoxName) {
        tester.assertCheckboxSelected(checkBoxName);
    }

    /**
     * Assert that a specific checkbox is selected.
     * 
     * @param checkBoxName
     * @param checkBoxValue
     */
    public void assertCheckboxSelected(String checkBoxName, String checkBoxValue) {
        tester.assertCheckboxSelected(checkBoxName, checkBoxValue);
    }

    /**
     * Assert that a specific checkbox is not selected.
     * 
     * @param checkBoxName
     */
    public void assertCheckboxNotSelected(String checkBoxName) {
        tester.assertCheckboxNotSelected(checkBoxName);
    }

    /**
     * Assert that a specific checkbox is not selected.
     * 
     * @param checkBoxName
     * @param checkBoxValue
     */
    public void assertCheckboxNotSelected(String checkBoxName, String checkBoxValue) {
        tester.assertCheckboxNotSelected(checkBoxName, checkBoxValue);
    }

    /**
     * Assert that a specific option is present in a radio group.
     * 
     * @param name radio group name.
     * @param radioOption option to test for.
     */
    public void assertRadioOptionPresent(String name, String radioOption) {
        tester.assertRadioOptionPresent(name, radioOption);
    }

    /**
     * Assert that a specific option is not present in a radio group.
     * 
     * @param name radio group name.
     * @param radioOption option to test for.
     */
    public void assertRadioOptionNotPresent(String name, String radioOption) {
        tester.assertRadioOptionNotPresent(name, radioOption);
    }

    /**
     * Assert that a specific option is selected in a radio group.
     * 
     * @param name radio group name.
     * @param radioOption option to test for selection.
     */
    public void assertRadioOptionSelected(String name, String radioOption) {
        tester.assertRadioOptionSelected(name, radioOption);
    }

    /**
     * Assert that a specific option is not selected in a radio group.
     * 
     * @param name radio group name.
     * @param radioOption option to test for selection.
     */
    public void assertRadioOptionNotSelected(String name, String radioOption) {
        tester.assertRadioOptionNotSelected(name, radioOption);
    }

    /**
     * Assert that given options are present in a select box (by label).
     * 
     * @param selectName name of the select element.
     * @param optionLabels option labels.
     */
    public void assertSelectOptionsPresent(String selectName, String[] optionLabels) {
        tester.assertSelectOptionsPresent(selectName, optionLabels);
    }

    /**
     * Assert that a specific option is present in a select box (by label).
     * 
     * @param selectName name of the select element.
     * @param optionLabel option label.
     */
    public void assertSelectOptionPresent(String selectName, String optionLabel) {
        tester.assertSelectOptionPresent(selectName, optionLabel);
    }

    /**
     * Assert that given options are present in a select box (by value).
     * 
     * @param selectName name of the select element.
     * @param optionValues option labels.
     */
    public void assertSelectOptionValuesPresent(String selectName, String[] optionValues) {
        tester.assertSelectOptionValuesPresent(selectName, optionValues);
    }

    /**
     * Assert that a specific option is present in a select box (by value).
     * 
     * @param selectName name of the select element.
     * @param optionValue option value.
     */
    public void assertSelectOptionValuePresent(String selectName, String optionValue) {
        tester.assertSelectOptionValuePresent(selectName, optionValue);
    }

    
    public void assertSelectOptionValueNotPresent(String selectName, String optionValue) {
        tester.assertSelectOptionValueNotPresent(selectName, optionValue);
    }

    /**
     * Assert that a specific option is not present in a select box.
     * 
     * @param selectName name of the select element.
     * @param expectedOption option label.
     */
    public void assertSelectOptionNotPresent(String selectName, String optionLabel) {
        tester.assertSelectOptionNotPresent(selectName, optionLabel);
    }

    /**
     * Assert that the display values of a select element's options match a given array of strings.
     * 
     * @param selectName name of the select element.
     * @param expectedOptions expected labels for the select box.
     */
    public void assertSelectOptionsEqual(String selectName, String[] expectedOptions) {
        tester.assertSelectOptionsEqual(selectName, expectedOptions);
    }

    /**
     * Assert that the display values of a select element's options do not match a given array of strings.
     * 
     * @param selectName name of the select element.
     * @param expectedOptions expected display values for the select box.
     */
    public void assertSelectOptionsNotEqual(String selectName, String[] expectedOptions) {
        tester.assertSelectOptionsNotEqual(selectName, expectedOptions);
    }

    /**
     * Assert that the values of a select element's options match a given array of strings.
     * 
     * @param selectName name of the select element.
     * @param expectedValues expected values for the select box.
     */
    public void assertSelectOptionValuesEqual(String selectName, String[] expectedValues) {
        tester.assertSelectOptionValuesEqual(selectName, expectedValues);
    }

    /**
     * Assert that the values of a select element's options do not match a given array of strings.
     * 
     * @param selectName name of the select element.
     * @param optionValues expected values for the select box.
     */
    public void assertSelectOptionValuesNotEqual(String selectName, String[] optionValues) {
        tester.assertSelectOptionValuesNotEqual(selectName, optionValues);
    }

    /**
     * Assert that the currently selected display label(s) of a select box matches given label(s).
     * 
     * @param selectName name of the select element.
     * @param labels expected display label(s) of the selected option.
     */
    public void assertSelectedOptionsEqual(String selectName, String[] labels) {
        tester.assertSelectedOptionsEqual(selectName, labels);
    }

    
    public void assertSelectedOptionEquals(String selectName, String option) {
        tester.assertSelectedOptionEquals(selectName, option);
    }

    /**
     * Assert that the currently selected value(s) of a select box matches given value(s).
     * 
     * @param selectName name of the select element.
     * @param values expected value(s) of the selected option.
     */
    public void assertSelectedOptionValuesEqual(String selectName, String[] values) {
        tester.assertSelectedOptionValuesEqual(selectName, values);
    }

    /**
     * Assert that the currently selected value of a select box matches given value.
     * 
     * @param selectName name of the select element.
     * @param value expected value of the selected option.
     */
    public void assertSelectedOptionValueEquals(String selectName, String value) {
        tester.assertSelectedOptionValueEquals(selectName, value);
    }

    /**
     * Assert that the currently selected display value(s) of a select box matches a given value(s).
     * 
     * @param selectName name of the select element.
     * @param regexps expected display value of the selected option.
     */
    public void assertSelectedOptionsMatch(String selectName, String[] regexps) {
        tester.assertSelectedOptionsMatch(selectName, regexps);
    }

    
    public void assertSelectedOptionMatches(String selectName, String regexp) {
        tester.assertSelectedOptionMatches(selectName, regexp);
    }

    /**
     * Assert that a submit button is present. <br/> A submit button can be the following HTML elements:
     * <ul>
     * <li>submit input
     * <li>image input
     * <li>submit button
     * </ul>
     * 
     */
    public void assertSubmitButtonPresent() {
        tester.assertSubmitButtonPresent();
    }

    /**
     * Assert that a submit button with a given name is present. <br/> A submit button can be the following HTML
     * elements:
     * <ul>
     * <li>submit input
     * <li>image input
     * <li>submit button
     * </ul>
     * 
     * @param buttonName
     */
    public void assertSubmitButtonPresent(String buttonName) {
        tester.assertSubmitButtonPresent(buttonName);
    }

    /**
     * Assert that no submit button is present in the current form. <br/> A submit button can be the following HTML
     * elements:
     * <ul>
     * <li>submit input
     * <li>image input
     * <li>submit button
     * </ul>
     * 
     * @param buttonName
     */
    public void assertSubmitButtonNotPresent() {
        tester.assertSubmitButtonNotPresent();
    }

    /**
     * Assert that a submit button with a given name is not present. <br/> A submit button can be the following HTML
     * elements:
     * <ul>
     * <li>submit input
     * <li>image input
     * <li>submit button
     * </ul>
     * 
     * @param buttonName
     */
    public void assertSubmitButtonNotPresent(String buttonName) {
        tester.assertSubmitButtonNotPresent(buttonName);
    }

    /**
     * Assert that a submit button with a given name and value is present. <br/> A submit button can be the following
     * HTML elements:
     * <ul>
     * <li>submit input
     * <li>image input
     * <li>submit button
     * </ul>
     * 
     * @param buttonName
     * @param buttonValue
     */
    public void assertSubmitButtonPresent(String buttonName, String buttonValue) {
        tester.assertSubmitButtonPresent(buttonName, buttonValue);
    }

    /**
     * Assert that a reset button is present. <br/> A reset button can be the following HTML elements:
     * <ul>
     * <li>reset input
     * <li>reset button
     * </ul>
     * 
     */
    public void assertResetButtonPresent() {
        tester.assertResetButtonPresent();
    }

    /**
     * Assert that a reset button with a given name is present.<br/> A reset button can be the following HTML elements:
     * <ul>
     * <li>reset input
     * <li>reset button
     * </ul>
     * 
     * @param buttonName
     */
    public void assertResetButtonPresent(String buttonName) {
        tester.assertResetButtonPresent(buttonName);
    }

    /**
     * Assert that no reset button is present in the current form.<br/> A reset button can be the following HTML
     * elements:
     * <ul>
     * <li>reset input
     * <li>reset button
     * </ul>
     * 
     * @param buttonName
     */
    public void assertResetButtonNotPresent() {
        tester.assertResetButtonNotPresent();
    }

    /**
     * Assert that a reset button with a given name is not present.<br/> A reset button can be the following HTML
     * elements:
     * <ul>
     * <li>reset input
     * <li>reset button
     * </ul>
     * 
     * @param buttonName
     */
    public void assertResetButtonNotPresent(String buttonName) {
        tester.assertResetButtonNotPresent(buttonName);
    }

    /**
     * Assert that a button with a given id is present in the current window.<br/> A button can be the following HTML
     * elements:
     * <ul>
     * <li>button input
     * <li>button button
     * </ul>
     * 
     * @param buttonId
     */
    public void assertButtonPresent(String buttonId) {
        tester.assertButtonPresent(buttonId);
    }

    /**
     * Assert that a button with a given text is present in the current window.
     * 
     * @param text Text representation of button content.
     */
    public void assertButtonPresentWithText(String text) {
        tester.assertButtonPresentWithText(text);
    }

    /**
     * Assert that a button with a given text is not present in the current window.
     * 
     * @param text Text representation of button content.
     */
    public void assertButtonNotPresentWithText(String text) {
        tester.assertButtonNotPresentWithText(text);
    }

    /**
     * Assert that a button with a given id is not present in the current window.
     * 
     * @param buttonId
     */
    public void assertButtonNotPresent(String buttonId) {
        tester.assertButtonNotPresent(buttonId);
    }

    /**
     * Assert that a link with a given id is present in the response.
     * 
     * @param linkId
     */
    public void assertLinkPresent(String linkId) {
        tester.assertLinkPresent(linkId);
    }

    /**
     * Assert that no link with the given id is present in the response.
     * 
     * @param linkId
     */
    public void assertLinkNotPresent(String linkId) {
        tester.assertLinkNotPresent(linkId);
    }

    /**
     * Assert that a link containing the supplied text is present.
     * 
     * @param linkText
     */
    public void assertLinkPresentWithText(String linkText) {
        tester.assertLinkPresentWithText(linkText);
    }

    /**
     * Assert that no link containing the supplied text is present.
     * 
     * @param linkText
     */
    public void assertLinkNotPresentWithText(String linkText) {
        tester.assertLinkNotPresentWithText(linkText);
    }

    /**
     * Assert that a link containing the supplied text is present.
     * 
     * @param linkText
     * @param index The 0-based index, when more than one link with the same text is expected.
     */
    public void assertLinkPresentWithText(String linkText, int index) {
        tester.assertLinkPresentWithText(linkText, index);
    }

    /**
     * Assert that no link containing the supplied text is present.
     * 
     * @param linkText
     * @param index The 0-based index, when more than one link with the same text is expected.
     */
    public void assertLinkNotPresentWithText(String linkText, int index) {
        tester.assertLinkNotPresentWithText(linkText, index);
    }

    /**
     * Assert that a link containing the Exact text is present.
     * 
     * @param linkText
     */
    public void assertLinkPresentWithExactText(String linkText) {
        tester.assertLinkPresentWithExactText(linkText);
    }

    /**
     * Assert that no link containing the Exact text is present.
     * 
     * @param linkText
     */
    public void assertLinkNotPresentWithExactText(String linkText) {
        tester.assertLinkNotPresentWithExactText(linkText);
    }

    /**
     * Assert that a link containing the Exact text is present.
     * 
     * @param linkText
     * @param index The 0-based index, when more than one link with the same text is expected.
     */
    public void assertLinkPresentWithExactText(String linkText, int index) {
        tester.assertLinkPresentWithExactText(linkText, index);
    }

    /**
     * Assert that no link containing the Exact text is present.
     * 
     * @param linkText
     * @param index The 0-based index, when more than one link with the same text is expected.
     */
    public void assertLinkNotPresentWithExactText(String linkText, int index) {
        tester.assertLinkNotPresentWithExactText(linkText, index);
    }

    /**
     * Assert that a link containing a specified image is present.
     * 
     * @param imageFileName A suffix of the image's filename; for example, to match <tt>"images/my_icon.png"</tt>,
     *            you could just pass in <tt>"my_icon.png"</tt>.
     */
    public void assertLinkPresentWithImage(String imageFileName) {
        tester.assertLinkPresentWithImage(imageFileName);
    }

    /**
     * Assert that a link containing a specified image is not present.
     * 
     * @param imageFileName A suffix of the image's filename; for example, to match <tt>"images/my_icon.png"</tt>,
     *            you could just pass in <tt>"my_icon.png"</tt>.
     */
    public void assertLinkNotPresentWithImage(String imageFileName) {
        tester.assertLinkNotPresentWithImage(imageFileName);
    }

    /**
     * Assert that an element with a given id is present.
     * 
     * @param anID element id to test for.
     */
    public void assertElementPresent(String anID) {
        tester.assertElementPresent(anID);
    }

    /**
     * Assert that an element with a given id is not present.
     * 
     * @param anID element id to test for.
     */
    public void assertElementNotPresent(String anID) {
        tester.assertElementNotPresent(anID);
    }

    /**
     * Assert that an element with a given xpath is present.
     * 
     * @param xpath element xpath to test for.
     */
    public void assertElementPresentByXPath(String xpath) {
        tester.assertElementPresentByXPath(xpath);
    }

    /**
     * Assert that an element with a given xpath is not present.
     * 
     * @param xpath element xpath to test for.
     */
    public void assertElementNotPresentByXPath(String xpath) {
        tester.assertElementNotPresentByXPath(xpath);
    }

    /**
     * Assert that a given element contains specific text.
     * 
     * @param elementID id of element to be inspected.
     * @param text to check for.
     */
    public void assertTextInElement(String elementID, String text) {
        tester.assertTextInElement(elementID, text);
    }

    
    public void assertTextNotInElement(String elementID, String text) {
        tester.assertTextNotInElement(elementID, text);
    }

    /**
     * Assert that a given element matches a specific regexp.
     * 
     * @param elementID id of element to be inspected.
     * @param regexp to match.
     */
    public void assertMatchInElement(String elementID, String regexp) {
        tester.assertMatchInElement(elementID, regexp);
    }

    /**
     * Assert that a given element does not match a specific regexp.
     * 
     * @param elementID id of element to be inspected.
     * @param regexp to match.
     */
    public void assertNoMatchInElement(String elementID, String regexp) {
        tester.assertNoMatchInElement(elementID, regexp);
    }

    /**
     * Assert that a window with the given name is open.
     * 
     * @param windowName
     */
    public void assertWindowPresent(String windowName) {
        tester.assertWindowPresent(windowName);
    }

    /**
     * Assert that a window with the given ID is open.
     * 
     * @param windowID Javascript window ID.
     */
    public void assertWindowPresent(int windowID) {
        tester.assertWindowPresent(windowID);
    }

    /**
     * Assert that at least one window with the given title is open.
     * 
     * @param title
     */
    public void assertWindowPresentWithTitle(String title) {
        tester.assertWindowPresentWithTitle(title);
    }

    /**
     * Assert that the number of opened windows equals given value.
     * 
     * @param windowCount Window count
     */
    public void assertWindowCountEquals(int windowCount) {
        tester.assertWindowCountEquals(windowCount);
    }

    /**
     * Assert that a frame with the given name or ID is present.
     * 
     * @param frameNameOrId Name or ID of the frame. ID is checked first.
     */
    public void assertFramePresent(String frameNameOrId) {
        tester.assertFramePresent(frameNameOrId);
    }

    /**
     * Checks to see if a cookie is present in the response.
     * 
     * @param cookieName The cookie name
     */
    public void assertCookiePresent(String cookieName) {
        tester.assertCookiePresent(cookieName);
    }

    /**
     * Check to see if a cookie has the given value.
     * 
     * @param cookieName The cookie name
     * @param expectedValue The cookie value
     */
    public void assertCookieValueEquals(String cookieName, String expectedValue) {
        tester.assertCookieValueEquals(cookieName, expectedValue);
    }

    /**
     * Check to see if a cookie value match the given regexp.
     * 
     * @param cookieName The cookie name
     * @param regexp The regexp
     */
    public void assertCookieValueMatch(String cookieName, String regexp) {
        tester.assertCookieValueMatch(cookieName, regexp);
    }

    /**
     * @deprecated
     */
    public String getFormElementValue(String formElementName) {
        return tester.getFormElementValue(formElementName);
    }

    /**
     * Begin interaction with a specified form. If form interaction methods are called without explicitly calling this
     * method first, jWebUnit will attempt to determine itself which form is being manipulated.
     * 
     * It is not necessary to call this method if their is only one form on the current page.
     * 
     * @param index 0-based index of the form to work with.
     */
    public void setWorkingForm(int index) {
        tester.setWorkingForm(index);
    }

    /**
     * Begin interaction with a specified form. If form interaction methods are called without explicitly calling this
     * method first, jWebUnit will attempt to determine itself which form is being manipulated.
     * 
     * It is not necessary to call this method if their is only one form on the current page.
     * 
     * @param nameOrId name or id of the form to work with.
     */
    public void setWorkingForm(String nameOrId) {
        tester.setWorkingForm(nameOrId);
    }

    /**
     * Begin interaction with a specified form. If form interaction methods are called without explicitly calling this
     * method first, jWebUnit will attempt to determine itself which form is being manipulated.
     * 
     * It is not necessary to call this method if their is only one form on the current page.
     * 
     * @param nameOrId name or id of the form to work with.
     * @param index The 0-based index, when more than one form with the same name is expected.
     */
    public void setWorkingForm(String nameOrId, int index) {
        tester.setWorkingForm(nameOrId, index);
    }

    /**
     * Set the value of a text or password input field.
     * 
     * @param inputName name of form element.
     * @param value value to set.
     */
    public void setTextField(String inputName, String value) {
        tester.setTextField(inputName, value);
    }

    /**
     * Select a specified checkbox. If the checkbox is already checked then the checkbox will stay checked.
     * 
     * @param checkBoxName name of checkbox to be selected.
     */
    public void checkCheckbox(String checkBoxName) {
        tester.checkCheckbox(checkBoxName);
    }

    /**
     * Select a specified checkbox. If the checkbox is already checked then the checkbox will stay checked.
     * 
     * @param checkBoxName name of checkbox to be selected.
     * @param value value of checkbox to be selected.
     */
    public void checkCheckbox(String checkBoxName, String value) {
        tester.checkCheckbox(checkBoxName, value);
    }

    /**
     * Deselect a specified checkbox. If the checkbox is already unchecked then the checkbox will stay unchecked.
     * 
     * @param checkBoxName name of checkbox to be deselected.
     */
    public void uncheckCheckbox(String checkBoxName) {
        tester.uncheckCheckbox(checkBoxName);
    }

    /**
     * Deselect a specified checkbox. If the checkbox is already unchecked then the checkbox will stay unchecked.
     * 
     * @param checkBoxName name of checkbox to be deselected.
     * @param value value of checkbox to be deselected.
     */
    public void uncheckCheckbox(String checkBoxName, String value) {
        tester.uncheckCheckbox(checkBoxName, value);
    }

    /**
     * Select options with given display labels in a select element.
     * 
     * @param selectName name of select element.
     * @param labels labels of options to be selected.
     */
    public void selectOptions(String selectName, String[] labels) {
        tester.selectOptions(selectName, labels);
    }

    /**
     * Select an option with a given display label in a select element.
     * 
     * @param selectName name of select element.
     * @param label label of option to be selected.
     */
    public void selectOption(String selectName, String label) {
        tester.selectOption(selectName, label);
    }

    /**
     * Select options with given values in a select element.
     * 
     * @param selectName name of select element.
     * @param values values of options to be selected.
     */
    public void selectOptionsByValues(String selectName, String[] values) {
        tester.selectOptionsByValues(selectName, values);
    }

    /**
     * Select an option with a given value in a select element.
     * 
     * @param selectName name of select element.
     * @param values values of options to be selected.
     */
    public void selectOptionByValue(String selectName, String value) {
        tester.selectOptionByValue(selectName, value);
    }

    /**
     * Submit form - default submit button will be used (unnamed submit button, or named button if there is only one on
     * the form.
     */
    public void submit() {
        tester.submit();
    }

    /**
     * Submit form by pressing named button.
     * 
     * @param buttonName Submit button name attribut value.
     */
    public void submit(String buttonName) {
        tester.submit(buttonName);
    }

    /**
     * Submit the form by pressing the named button with the given value (label). Useful if you have more than one
     * submit button with same name.
     * 
     * @param buttonName Submit button name attribut value.
     * @param buttonValue Submit button value attribut value.
     */
    public void submit(String buttonName, String buttonValue) {
        tester.submit(buttonName, buttonValue);
    }

    /**
     * Reset the current form using the default reset button. See {@link #getForm}for an explanation of how the current
     * form is established.
     */
    public void reset() {
        tester.reset();
    }

    /**
     * Navigate by selection of a link containing given text.
     * 
     * @param linkText Text in the link.
     */
    public void clickLinkWithText(String linkText) {
        tester.clickLinkWithText(linkText);
    }

    /**
     * Navigate by selecting Nth link containing given text.
     * 
     * @param linkText Text in the link.
     * @param index The 0-based index, when more than one link with the same text is expected.
     */
    public void clickLinkWithText(String linkText, int index) {
        tester.clickLinkWithText(linkText, index);
    }

    /**
     * Navigate by selection of a link with the exact given text.
     * 
     * @param linkText Text of the link.
     */
    public void clickLinkWithExactText(String linkText) {
        tester.clickLinkWithExactText(linkText);
    }

    /**
     * Navigate by selecting Nth link with the exact given text.
     * 
     * @param linkText Text of the link.
     * @param index The 0-based index, when more than one link with the same text is expected.
     */
    public void clickLinkWithExactText(String linkText, int index) {
        tester.clickLinkWithExactText(linkText, index);
    }

    /**
     * Click the button with the given id.
     * 
     * @param buttonId Button ID attribut value.
     */
    public void clickButton(String buttonId) {
        tester.clickButton(buttonId);
    }

    /**
     * Clicks a button with <code>text</code> of the value attribute.
     * 
     * @param buttonValueText The text of the button (contents of the value attribute).
     */
    public void clickButtonWithText(String buttonValueText) {
        tester.clickButtonWithText(buttonValueText);
    }

    /**
     * Navigate by selection of a link with a given image.
     * 
     * @param imageFileName A suffix of the image's filename; for example, to match <tt>"images/my_icon.png"</tt>,
     *            you could just pass in <tt>"my_icon.png"</tt>.
     */
    public void clickLinkWithImage(String imageFileName) {
        tester.clickLinkWithImage(imageFileName);
    }

    /**
     * Navigate by selection of a link with given id.
     * 
     * @param linkId id of link
     */
    public void clickLink(String linkId) {
        tester.clickLink(linkId);
    }

    /**
     * Clicks a radio option. Asserts that the radio option exists first. *
     * 
     * @param radioGroup name of the radio group.
     * @param radioOption value of the option to check for.
     */
    public void clickRadioOption(String radioGroup, String radioOption) {
        tester.clickRadioOption(radioGroup, radioOption);
    }

    /**
     * Click element with given xpath.
     * 
     * @param xpath xpath of the element.
     */
    public void clickElementByXPath(String xpath) {
        tester.clickElementByXPath(xpath);
    }

    /**
     * Get the attribut value of the gicen element. For example, if you have img src="bla.gif" alt="toto",
     * getElementAttributByXPath("//img[@src='bla.gif']", "alt") returns "toto"
     * 
     * @param xpath XPath of the element.
     * @param attribut Name of the attribut.
     * @return The value of the attribut.
     */
    public String getElementAttributByXPath(String xpath, String attribut) {
        return tester.getElementAttributByXPath(xpath, attribut);
    }

    /**
     * Make a given window active.
     * 
     * @param windowName Name of the window.
     */
    public void gotoWindow(String windowName) {
        tester.gotoWindow(windowName);
    }

    /**
     * Make a given window active.
     * 
     * @param windowID Javascript ID of the window
     */
    public void gotoWindow(int windowID) {
        tester.gotoWindow(windowID);
    }

    /**
     * Make the root window active.
     */
    public void gotoRootWindow() {
        tester.gotoRootWindow();
    }

    /**
     * Make first window with the given title active.
     * 
     * @param title Title of the window.
     */
    public void gotoWindowByTitle(String title) {
        tester.gotoWindowByTitle(title);
    }

    /**
     * Make the given frame active.
     * 
     * @param frameNameOrId Name or ID of the frame. ID is checked first.
     */
    public void gotoFrame(String frameNameOrId) {
        tester.gotoFrame(frameNameOrId);
    }

    /**
     * Go to the given page like if user has typed the URL manually in the browser. Use
     * {@link TestContext#setBaseUrl(String) getTestContext().setBaseUrl(String)} to define base URL. Absolute URL
     * should start with "http://", "https://" or "www.".
     * 
     * @param url absolute or relative URL (relative to base URL).
     */
    public void gotoPage(String url) {
        tester.gotoPage(url);
    }

    /**
     * Print all the cookies to stdout.
     * 
     */
    public void dumpCookies() {
        tester.dumpCookies();
    }

    /**
     * Get the source of the HTML page (like in a real browser), or HTTP body for a non HTML content.
     * 
     * @return The HTML content.
     */
    public String getPageSource() {
        return tester.getPageSource();
    }

    /**
     * Get the last data sent by the server.
     * 
     * @return HTTP server response.
     */
    public String getServeurResponse() {
        return tester.getServeurResponse();
    }

    /**
     * Save the last downloaded page (or file) to the disk.
     * 
     * @param f The file name.
     */
    public void saveAs(File f) {
        tester.saveAs(f);
    }

    /**
     * Download the current page (or file) and compare it with the given file.
     * 
     * @param expected Expected file URL.
     */
    public void assertDownloadedFileEquals(URL expected) {
        tester.assertDownloadedFileEquals(expected);
    }

    /**
     * Dump html of current response to System.out - for debugging purposes.
     * 
     * @param stream
     * @deprecated Use {@link WebTester#getPageSource()}
     */
    public void dumpHtml() {
        tester.dumpHtml();
    }

    /**
     * Dump html of current response to a specified stream - for debugging purposes.
     * 
     * @param stream
     * @deprecated Use {@link WebTester#getPageSource()}
     */
    public void dumpHtml(PrintStream stream) {
        tester.dumpHtml(stream);
    }

    /**
     * Dump the table as the 2D array that is used for assertions - for debugging purposes.
     * 
     * @param tableNameOrId
     * @param stream
     */
    public void dumpTable(String tableNameOrId) {
        tester.dumpTable(tableNameOrId);
    }

    /**
     * Dump the table as the 2D array that is used for assertions - for debugging purposes.
     * 
     * @param tableNameOrId
     * @param table
     * @param stream
     */
    public void dumpTable(String tableNameOrId, PrintStream stream) {
        tester.dumpTable(tableNameOrId, stream);
    }

    /**
     * Enable or disable Javascript support
     */
    public void setScriptingEnabled(boolean value) {
        tester.setScriptingEnabled(value);
    }

    /**
     * Set the Testing Engine that you want to use for the tests based on the Testing Engine Key.
     * 
     * @see TestingEngineRegistry
     * @param testingEngineKey The testingEngineKey to set.
     */
    public void setTestingEngineKey(String testingEngineKey) {
        tester.setTestingEngineKey(testingEngineKey);
    }

    /**
     * Gets the Testing Engine Key that is used to find the proper testing engine class (HtmlUnitDialog /
     * SeleniumDialog) for the tests.
     * 
     * @return Returns the testingEngineKey.
     */
    public String getTestingEngineKey() {
        return tester.getTestingEngineKey();
    }

    /**
     * Set the value of a form input element.
     * 
     * @param formElementName name of form element.
     * @param value
     * @deprecated use setTextField or other methods
     */
    public void setFormElement(String formElementName, String value) {
        tester.setFormElement(formElementName, value);
    }

    /**
     * Tell that the given alert boxe is expected.
     * 
     * @param message Message in the alert.
     */
    public void setExpectedJavaScriptAlert(String message) {
        tester.setExpectedJavaScriptAlert(message);
    }

    /**
     * Tell that the given alert boxes are expected in the given order.
     * 
     * @param messages Messages in the alerts.
     */
    public void setExpectedJavaScriptAlert(String[] messages) {
        tester.setExpectedJavaScriptAlert(messages);
    }

    /**
     * Tell that the given confirm boxe is expected.
     * 
     * @param message Message in the confirm.
     * @param action Whether we should click on "OK" (true) or "Cancel" (false)
     */
    public void setExpectedJavaScriptConfirm(String message, boolean action) {
        tester.setExpectedJavaScriptConfirm(message, action);
    }

    /**
     * Tell that the given confirm boxes are expected in the given order.
     * 
     * @param messages Messages in the confirms.
     * @param actions Whether we should click on "OK" (true) or "Cancel" (false)
     */
    public void setExpectedJavaScriptConfirm(String[] messages, boolean[] actions) {
        tester.setExpectedJavaScriptConfirm(messages, actions);
    }

    /**
     * Tell that the given prompt boxe is expected.
     * 
     * @param message Message in the prompt.
     * @param input What we should put in the prompt (null if user press Cancel)
     */
    public void setExpectedJavaScriptPrompt(String message, String input) {
        tester.setExpectedJavaScriptPrompt(message, input);
    }

    /**
     * Tell that the given prompt boxes are expected in the given order.
     * 
     * @param messages Messages in the prompts.
     * @param inputs What we should put in the prompt (null if user press Cancel)
     */
    public void setExpectedJavaScriptPrompt(String[] messages, String[] inputs) {
        tester.setExpectedJavaScriptPrompt(messages, inputs);
    }

}
